float4x4 World;
float4x4 View;
float4x4 Projection;

float3 cameraPosition;

//light properties
float3 lightPosition;
float4 ambientLightColor;
float4 diffuseLightColor;
float4 specularLightColor;

//material properties
float specularPower;
float specularIntensity;


struct VertexShaderInput
{
    float4 Position : POSITION0;
	float3 Normal : NORMAL;
};

struct VertexShaderOutput
{
    float4 Position : POSITION0;
	float4 Color : COLOR;
};

VertexShaderOutput VertexShaderFunction(VertexShaderInput input)
{
    VertexShaderOutput output;

    float4 worldPosition = mul(input.Position, World);
    float4 viewPosition = mul(worldPosition, View);
    output.Position = mul(viewPosition, Projection);

	float3 normal = mul(float4(input.Normal, 0), World);
	normal = normalize(normal);

	float3 directionToLight = normalize(lightPosition - worldPosition);
	float diffuseIntensity = saturate( dot(directionToLight, normal));
	float4 diffuse = diffuseLightColor * diffuseIntensity;

	//calculate Phong components per-pixel
	float3 reflectionVector = normalize(reflect(-directionToLight, normal));
	float3 directionToCamera = normalize(cameraPosition - worldPosition);

	//calculate specular component
	float4 specular = specularLightColor * specularIntensity * 
				   pow(saturate(dot(reflectionVector, directionToCamera)), 
					   specularPower);

	//all color components are summed in the pixel shader
	output.Color = specular  + diffuse + ambientLightColor;
	output.Color.a = 1.0;

    return output;
}

float4 PixelShaderFunction(VertexShaderOutput input) : COLOR0
{
    return input.Color;
}

technique Technique1
{
    pass Pass1
    {
        // TODO: set renderstates here.

        VertexShader = compile vs_1_1 VertexShaderFunction();
        PixelShader = compile ps_1_1 PixelShaderFunction();
    }
}
