using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace ShadersTest
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        KeyboardState lastKbState;

        Model[] invaders = new Model[3];
        Effect[] effects = new Effect[3];

        Matrix[] world = new Matrix[3];
        Matrix view, projection;
        float theta = 0, phi = 0;

        Texture2D tex;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // TODO: use this.Content to load your game content here
            tex = Content.Load<Texture2D>("7042-v7");

            invaders[0] = Content.Load<Model>("Invader1");
            invaders[1] = Content.Load<Model>("Invader2");
            invaders[2] = Content.Load<Model>("Invader3");

            effects[0] = Content.Load<Effect>("shader1");
            effects[1] = Content.Load<Effect>("shader2");
            effects[2] = Content.Load<Effect>("shader3");

            for (int i = 0; i < 3; ++i)
                ChangeEffect(invaders[i], effects[i]);

            Vector3 cameraPos = Vector3.UnitZ * 80;
            view = Matrix.CreateLookAt(cameraPos, Vector3.Zero, Vector3.Up);
            float aspectRatio = (float)graphics.GraphicsDevice.Viewport.Width / (float)graphics.GraphicsDevice.Viewport.Height;
            projection = Matrix.CreatePerspectiveFieldOfView(MathHelper.PiOver4, aspectRatio, 0.1f, 100);

            for (int i = 0; i < 3; ++i)
            {
                effects[i].Parameters["View"].SetValue(view);
                effects[i].Parameters["Projection"].SetValue(projection);
            }

            SetLightning(effects[0], cameraPos, new Vector3(5 - 26, 5, 5));
            SetLightning(effects[1], cameraPos, new Vector3(5, 5, 5));
            SetLightning(effects[2], cameraPos, new Vector3(5 + 26, 5, 5));
            effects[2].Parameters["ModelTexture"].SetValue(tex);
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            KeyboardState kbState = Keyboard.GetState();
            // Allows the game to exit
            if (kbState.IsKeyDown(Keys.Escape))
                this.Exit();

            if (kbState.IsKeyDown(Keys.Left))
                phi += (float)(gameTime.ElapsedGameTime.TotalSeconds);
            else if (kbState.IsKeyDown(Keys.Right))
                phi -= (float)(gameTime.ElapsedGameTime.TotalSeconds);

            if (kbState.IsKeyDown(Keys.Up))
                theta += (float)(gameTime.ElapsedGameTime.TotalSeconds);
            else if (kbState.IsKeyDown(Keys.Down))
                theta -= (float)(gameTime.ElapsedGameTime.TotalSeconds);

            lastKbState = kbState;

            world[1] = Matrix.CreateRotationX(theta) * Matrix.CreateRotationY(phi);
            world[0] = world[1] * Matrix.CreateTranslation(-26, 0, 0);
            world[2] = world[1] * Matrix.CreateTranslation(26, 0, 0);

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            DrawModel(invaders[0], world[0]);
            DrawModel(invaders[1], world[1]);
            DrawModel(invaders[2], world[2]);

            base.Draw(gameTime);
        }

        private void DrawModel(Model model, Matrix world)
        {
            Matrix[] transforms = new Matrix[model.Bones.Count];
            model.CopyAbsoluteBoneTransformsTo(transforms);

            foreach (ModelMesh mesh in model.Meshes)
            {
                foreach (Effect effect in mesh.Effects)
                {
                    effect.Parameters["World"].SetValue(transforms[mesh.ParentBone.Index] * world);
                }
                mesh.Draw();
            }
        }

        public void ChangeEffect(Model model, Effect effect)
        {
            foreach (ModelMesh mesh in model.Meshes)
            {
                foreach (ModelMeshPart part in mesh.MeshParts)
                {
                    part.Effect = effect;
                }
            }
        }

        public void SetLightning(Effect effect, Vector3 cameraPos, Vector3 lightPos)
        {
            effect.Parameters["cameraPosition"].SetValue(cameraPos);
            effect.Parameters["lightPosition"].SetValue(lightPos);
            effect.Parameters["ambientLightColor"].SetValue(new Vector4(0.1f, 0.1f, 0.1f, 1));
            effect.Parameters["diffuseLightColor"].SetValue(new Vector4(0.0f, 0.5f, 0.0f, 1));
            effect.Parameters["specularLightColor"].SetValue(new Vector4(1.0f, 1.0f, 1.0f, 1));
            effect.Parameters["specularPower"].SetValue(50);
            effect.Parameters["specularIntensity"].SetValue(1);
        }
    }
}
